#!/bin/bash

set -e

LD=ld
DKMS_BUILD_DIR="$( cd -- "$(dirname "$0")" > /dev/null 2>&1 ; pwd -P)"
PROJECT_DIR=$DKMS_BUILD_DIR/../..
TARGET=$PROJECT_DIR/dkms_target
TARGET_APU=$PROJECT_DIR/dkms_target_apu
DKMS_VERSION=${1:-'1.0.0'}
PACKAGE_TYPE=${2:-'deb'}
GPU_RPM_PACKAGE_NAME=$(cat ${DKMS_BUILD_DIR}/dkms.conf | grep "PACKAGE_NAME" | awk -F '"' '{print $2}')
APU_RPM_PACKAGE_NAME=$(echo $GPU_RPM_PACKAGE_NAME | sed "s/gpu/apu/")
DKMS_RELEASE_VERSION=${DKMS_VERSION}

FIND_FILTER=("-not" "-path" "*rogue/*" \
	"-not" "-path" "*innosrvkm/bridge/*" \
	"-not" "-path" "*rgx_nohw/*" \
	"-not" "-path" "*rgx_linux_plato/*" \
	"-not" "-path" "*rgx_linux_tc/*" \
	"-not" "-path" "*plato/*" \
	"-not" "-path" "*rgx_vexpress/*" \
	"-not" "-path" "*vz_example/*")

OPEN_SRCS_SUFFIX=_open_srcs.txt
BIN_EXCLUDE_OBJ_SUFFIX=_bin_exclude_objs.tmp

function prepare_env_and_make() {
	cd $PROJECT_DIR
	make -j16
	make export_config
	sed -i 's/^/export /' $DKMS_BUILD_DIR/modules_config.sh
	source $DKMS_BUILD_DIR/modules_config.sh

	if [[ -n $CROSS_COMPILE ]]; then
		LD=${CROSS_COMPILE}ld
	fi
}

function generate_kbuild() {

	if [[ $CONFIG_DRM_INNO_VPU != "y" ]]; then
		sed -i '/innovpu/Id' $TARGET/Kbuild
	fi

	if [[ $CONFIG_INNO_ION != "y" ]]; then
		sed -i '/innoion/Id' $TARGET/Kbuild
	fi

	if [[ $CONFIG_DRM_INNO_PMBUS != "y" ]]; then
		sed -i '/innopmbus/Id' $TARGET/Kbuild
	fi

	if [[ $CONFIG_DRM_INNO_POWER != "y" ]]; then
		sed -i '/innopower/Id' $TARGET/Kbuild
	fi

	if [[ $CONFIG_DRM_INNO_SR != "y" ]]; then
		sed -i '/innosr[^v]/Id' $TARGET/Kbuild
	fi

	sed -i '/CONFIG_DRM_INNO_APU/Id' $TARGET/Kbuild

	if [[ $CONFIG_DRM_INNO_DEBUG != "y" ]]; then
		sed -i '/tools/Id' $TARGET/Kbuild
		sed -i '/innogpu_info/Id' $TARGET/Kbuild
	fi

	sed -i 's/$(CONFIG_DRM_INNO_GPU)/m/' $TARGET/Kbuild

	if [[ $CONFIG_DRM_INNO_APU = "y" ]]; then
		sed -i 's/$(src)/\/usr\/src\/innogpu-kernel-'"${DKMS_VERSION}"'/' $TARGET_APU/Kbuild
		sed -i '2i\KBUILD_EXTRA_SYMBOLS += /usr/src/innogpu-kernel-'"${DKMS_VERSION}"'/Module.symvers' $TARGET_APU/Kbuild
		sed -i '/innovpu/Id' $TARGET_APU/Kbuild
		sed -i '/innoion/Id' $TARGET_APU/Kbuild
		sed -i '/innopmbus/Id' $TARGET_APU/Kbuild
		sed -i '/innopower/Id' $TARGET_APU/Kbuild
		sed -i '/innosr[^v]/Id' $TARGET_APU/Kbuild
		sed -i '/tools/Id' $TARGET_APU/Kbuild
		sed -i '/innogpu_info/Id' $TARGET_APU/Kbuild
		sed -i '/innogpu-y/Id' $TARGET_APU/Kbuild
		sed -i '/CONFIG_DRM_INNO_GPU/Id' $TARGET_APU/Kbuild
		sed -i 's/$(CONFIG_DRM_INNO_APU)/m/' $TARGET_APU/Kbuild
	fi
}

# 模块是否有闭源的二进制
function module_has_private_objs() {
	local module=$1
	objs_dir=$PROJECT_DIR/${module}/
	bin_exclude_objs=$DKMS_BUILD_DIR/${module}${BIN_EXCLUDE_OBJ_SUFFIX}

	find $objs_dir -name "*.o" | grep -vFf $bin_exclude_objs | wc -l
}

# 生成链接二进制需要排除的.o文件
# 该函数是通用实现,不要轻易修改,如果模块有特殊需求自行实现
function prepare_bin_exclude_objs() {
	local module=$1
	open_srcs=$DKMS_BUILD_DIR/${module}${OPEN_SRCS_SUFFIX}
	bin_exclude_objs=$DKMS_BUILD_DIR/${module}${BIN_EXCLUDE_OBJ_SUFFIX}

	cat $open_srcs | grep "\.c" > $bin_exclude_objs
	sed -i 's/\.c/.o/' $bin_exclude_objs
}

# 生成编译开源模块的makefile文件
# 该函数是通用实现,不要轻易修改,如果模块有特殊需求自行实现
function generate_makefile() {
	local module=$1
	makefile=$TARGET/${module}/Makefile
	cnt=$(module_has_private_objs $module)
	cp $DKMS_BUILD_DIR/${module}${BIN_EXCLUDE_OBJ_SUFFIX} $makefile

	sed -i 's/^/'"${module}"'\//' $makefile
	test -s $makefile && sed -i '1i\'"${module^^}"'_OBJS =' $makefile || \
		echo "${module^^}_OBJS =" >> $makefile
	sed -i 's/$/ \\/' $makefile
	if [[ $cnt != 0 ]]; then
		sed -i '$a\'"${module}"'/'"${module}"'.o' $makefile
	fi
	sed -i '$a\\nifeq ($(SEPARATE_BUILD),y)' $makefile
	sed -i '$a\  '"${module}"'-y := $('"${module^^}"'_OBJS)' $makefile
	sed -i '$a\  obj-m += '"${module}"'.o' $makefile
	sed -i '$a\else' $makefile
	sed -i '$a\  '"${module^^}"' := $('"${module^^}"'_OBJS)' $makefile
	sed -i '$a\endif' $makefile
	sed -i '1i\'"${module^^}"'_INCLUDE :=-I$(INNOGPU_PRO_PATH)/'"${module}"'' $makefile
}

function generate_innoapu_makefile() {
	local module=innoapu
	local submodule_array=("img_mem" "vha")

	for submodule in "${submodule_array[@]}"
	do
		makefile=$TARGET_APU/${module}/${submodule}/Makefile
		#cnt=$(module_has_private_objs "${module}_${submodule}")
		objs_dir=$PROJECT_DIR/${module}/${submodule}/
		bin_exclude_objs=$DKMS_BUILD_DIR/${module}_${submodule}${BIN_EXCLUDE_OBJ_SUFFIX}
		cnt=$(find $objs_dir -name "*.o" -not -name "${submodule}".o -not -name "${submodule}".mod.o | grep -vFf $bin_exclude_objs | wc -l)

		cp $DKMS_BUILD_DIR/${module}_${submodule}${BIN_EXCLUDE_OBJ_SUFFIX} $makefile

		#sed -i 's/^/'"${module}"'\//' $makefile
		test -s $makefile && sed -i 's/^/'"${submodule}"'-y += /' $makefile || \
			echo "${submodule}-y +=" >> $makefile
		sed -i '1s/+=/:=/' $makefile
		if [[ $cnt != 0 ]]; then
			sed -i '$a\'"${submodule}"'-y += '"${submodule}"'_part.o' $makefile
		fi
		sed -i '$a\obj-m += '"${submodule}"'.o' $makefile
	done

	sed -i '1i\ccflags-y += -DOSID=0' $TARGET_APU/innoapu/img_mem/Makefile
	sed -i '1i\ccflags-y += -DDEFAULT_SYMBOL_NAMESPACE=IMG_MEM' $TARGET_APU/innoapu/img_mem/Makefile

	sed -i '1i\ccflags-y += -include config_kernel.h' $TARGET_APU/innoapu/vha/Makefile
	sed -i '1i\ccflags-y += -DCFG_SYS_AURA' $TARGET_APU/innoapu/vha/Makefile
	sed -i '1i\ccflags-y += -DDEFAULT_SYMBOL_NAMESPACE=VHA_CORE' $TARGET_APU/innoapu/vha/Makefile
	sed -i '1i\ccflags-y += -DCONFIG_VHA_G1' $TARGET_APU/innoapu/vha/Makefile
	sed -i '1i\ccflags-y += -DCONFIG_VHA_G0' $TARGET_APU/innoapu/vha/Makefile
	sed -i '1i\ccflags-y += -DVHA_MMU_MIRRORED_CTX_SUPPORT' $TARGET_APU/innoapu/vha/Makefile
	sed -i '1i\ccflags-y += -DHW_AX3' $TARGET_APU/innoapu/vha/Makefile
	sed -i '1i\ccflags-y += -DVHA_ENHANCED_APM' $TARGET_APU/innoapu/vha/Makefile
	sed -i '1i\ccflags-y += -DOSID=0' $TARGET_APU/innoapu/vha/Makefile

	touch $TARGET_APU/${module}/Makefile
	makefile=$TARGET_APU/${module}/Makefile
	echo 'subdir-ccflags-y += -I$(src)/include' > $makefile
	sed -i '$a\subdir-ccflags-y += -I$(src)/../innogpu' $makefile
	sed -i '$a\subdir-ccflags-y += -I$(src)/../innodma' $makefile
	sed -i '$a\subdir-ccflags-y += -DCONFIG_VHA_IOCTL_DMA' $makefile
	sed -i '$a\obj-m += vha/ img_mem/' $makefile
}

# 生成二进制
# 该函数是通用实现,如果模块有特殊需求自行实现,参照innosrvkm
function generate_bin() {
	local module=$1
	objs_dir=$PROJECT_DIR/${module}/
	bin_name=$TARGET/${module}/${module}.o_shipped
	bin_exclude_objs=$DKMS_BUILD_DIR/${module}${BIN_EXCLUDE_OBJ_SUFFIX}
	cnt=$(module_has_private_objs $module)

	if [[ $cnt != 0 ]]; then
		$LD -r -o $bin_name `find $objs_dir -name "*.o" | \
			grep -vFf $bin_exclude_objs`
	fi
}

# 拷贝开源的代码
# 该函数是通用实现,如果模块有特殊需求自行实现,参照innosrvkm
function copy_open_srcs() {
	local module=$1
	src_dir=$PROJECT_DIR/${module}
	dst_dir=$TARGET/${module}
	open_srcs=$DKMS_BUILD_DIR/${module}${OPEN_SRCS_SUFFIX}

	mkdir -p $dst_dir

	for i in $(cat $open_srcs); do
		find $src_dir -name $i -exec cp -v {} $dst_dir \;
	done
}

function prepare_innogpu_exclude_objs() {
	open_srcs=$DKMS_BUILD_DIR/innogpu${OPEN_SRCS_SUFFIX}
	bin_exclude_objs=$DKMS_BUILD_DIR/innogpu${BIN_EXCLUDE_OBJ_SUFFIX}

	if [[ $CONFIG_INNO_ION != "y" ]]; then
		sed '/ionn*/d' $open_srcs | grep "\.c" > $bin_exclude_objs
	else
		cat $open_srcs | grep "\.c" > $bin_exclude_objs
	fi

	cat $open_srcs | grep "\.S" >> $bin_exclude_objs

	sed -i 's/\.c/.o/' $bin_exclude_objs
	sed -i 's/\.S/.o/' $bin_exclude_objs
}

function prepare_innodma_exclude_objs() {
	prepare_bin_exclude_objs innodma
}

function prepare_innoion_exclude_objs() {
	prepare_bin_exclude_objs innoion
}

function prepare_innopmbus_exclude_objs() {
	prepare_bin_exclude_objs innopmbus
}

function prepare_innovpu_exclude_objs() {
	prepare_bin_exclude_objs innovpu
}

function prepare_innosr_exclude_objs() {
	prepare_bin_exclude_objs innosr
}

function prepare_innopower_exclude_objs() {
	prepare_bin_exclude_objs innopower
}

function prepare_innoapu_exclude_objs() {
	prepare_bin_exclude_objs innoapu_img_mem
	prepare_bin_exclude_objs innoapu_vha
}

function prepare_innosrvkm_exclude_objs() {
	open_srcs=$DKMS_BUILD_DIR/innosrvkm_open_srcs.txt
	bin_objs=$DKMS_BUILD_DIR/innosrvkm${BIN_EXCLUDE_OBJ_SUFFIX}
	_FILE=${open_srcs}.tmp
	cp $open_srcs $_FILE

	if [[ $PVR_BUILD_DIR = android_container ]]; then
		sed -i '/pvr_sync_ioctl_drm.c/d' $_FILE
	else
		sed -i '/pvr_sync_ioctl_dev.*/d' $_FILE
		sed -i '/ion_sys.*/d' $_FILE
	fi

	if [[ $CONFIG_DRM_INNO_POWER != "y" ]]; then
		sed -i '/pvr_dvfs_device.*/d' $_FILE
	fi

	cat $_FILE | grep "\.c" > $bin_objs

	if [[ $CONFIG_DRM_INNO_DPU = "y" ]]; then
		dpu_open_srcs=$DKMS_BUILD_DIR/innodpu${OPEN_SRCS_SUFFIX}
		cat $dpu_open_srcs | grep "\.c" >> $bin_objs
		echo "innogpu_drm.c" >> $bin_objs

		if [[ $CONFIG_DRM_INNO_AUDIO = "y" ]]; then
			audio_open_srcs=$DKMS_BUILD_DIR/innoaudio${OPEN_SRCS_SUFFIX}
			cat $audio_open_srcs | grep "\.c" >> $bin_objs
		fi

	else
		echo "innogpu_drm_without_dpu.c" >> $bin_objs
	fi

	sed -i 's/\.c/.o/' $bin_objs
}

function generate_makefile_innogpu() {
	generate_makefile innogpu

	# innogpu不需要独立编译的控制
	sed -i '/SEPARATE/,+3d' $TARGET/innogpu/Makefile
	sed -i '/endif/d' $TARGET/innogpu/Makefile
}

function generate_makefile_innodma() {
	generate_makefile innodma
}

function generate_makefile_innoion() {
	generate_makefile innoion
}

function generate_makefile_innopmbus() {
	generate_makefile innopmbus
}

function generate_makefile_innosrvkm() {
	generate_makefile innosrvkm
	sed -i '1d' $TARGET/innosrvkm/Makefile
	sed -i '1i\'"${module^^}"'_INCLUDE :=-I$(INNOGPU_PRO_PATH)/'"${module}"'/include' \
		$TARGET/innosrvkm/Makefile
}

function generate_makefile_innopower() {
	generate_makefile innopower
}

function generate_makefile_innovpu() {
	generate_makefile innovpu
}

function generate_makefile_innosr() {
	generate_makefile innosr
}

function generate_makefile_innoapu() {
	generate_innoapu_makefile
}

function copy_innogpu_open_srcs() {
	copy_open_srcs innogpu
}

function copy_innodma_open_srcs() {
	copy_open_srcs innodma
}

function copy_innoion_open_srcs() {
	copy_open_srcs innoion
	cp -r $PROJECT_DIR/innoion/include $TARGET/innoion/include
}

function copy_innopmbus_open_srcs() {
	copy_open_srcs innopmbus
}

function copy_innosrvkm_open_srcs() {
	mkdir -p $TARGET/innosrvkm && mkdir -p $TARGET/innosrvkm/include
	cp $PROJECT_DIR/innosrvkm/include/config_kernel.h $TARGET/innosrvkm/include
	cp $PROJECT_DIR/innosrvkm/include/innoversion.h $TARGET/innosrvkm/include

	for i in $(cat $DKMS_BUILD_DIR/innosrvkm${OPEN_SRCS_SUFFIX}.tmp)
	do
		if [[ $i =~ "/" ]]; then
			DIR=${i%/*}
			FILE=${i#*/}
			mkdir -p $TARGET/innosrvkm/include/$DIR
			find $PROJECT_DIR/innosrvkm -name $FILE "${FIND_FILTER[@]}"  -exec cp -v {} $TARGET/innosrvkm/include/$DIR \;
		elif [[ $i =~ ".c" ]]; then
			find $PROJECT_DIR/innosrvkm -name $i "${FIND_FILTER[@]}" -exec cp -v {} $TARGET/innosrvkm \;
		else
			find $PROJECT_DIR/innosrvkm -name $i "${FIND_FILTER[@]}" -exec cp -v {} $TARGET/innosrvkm/include \;
		fi
	done

	if [[ $CONFIG_DRM_INNO_DPU = "y" ]]; then
		for i in $(cat $DKMS_BUILD_DIR/innodpu${OPEN_SRCS_SUFFIX})
		do
			if [[ $i =~ "/" ]]; then
				DIR=${i%/*}
				FILE=${i#*/}
				mkdir -p $TARGET/innosrvkm/include/$DIR
				find $PROJECT_DIR/innosrvkm -name $FILE "${FIND_FILTER[@]}"  -exec cp -v {} $TARGET/innosrvkm/include/$DIR \;
			elif [[ $i =~ ".c" ]]; then
				find $PROJECT_DIR/innosrvkm -name $i "${FIND_FILTER[@]}" -exec cp -v {} $TARGET/innosrvkm \;
			else
				find $PROJECT_DIR/innosrvkm -name $i "${FIND_FILTER[@]}" -exec cp -v {} $TARGET/innosrvkm/include \;
			fi
		done
		cp -v $PROJECT_DIR/innosrvkm/services/server/env/linux/innogpu_drm.c $TARGET/innosrvkm/innogpu_drm.c

		if [[ $CONFIG_DRM_INNO_AUDIO = "y" ]]; then
			for i in $(cat $DKMS_BUILD_DIR/innoaudio${OPEN_SRCS_SUFFIX})
			do
				if [[ $i =~ "/" ]]; then
					DIR=${i%/*}
					FILE=${i#*/}
					mkdir -p $TARGET/innosrvkm/include/$DIR
					find $PROJECT_DIR/innosrvkm -name $FILE "${FIND_FILTER[@]}"  -exec cp -v {} $TARGET/innosrvkm/include/$DIR \;
				elif [[ $i =~ ".c" ]]; then
					find $PROJECT_DIR/innosrvkm -name $i "${FIND_FILTER[@]}" -exec cp -v {} $TARGET/innosrvkm \;
				else
					find $PROJECT_DIR/innosrvkm -name $i "${FIND_FILTER[@]}" -exec cp -v {} $TARGET/innosrvkm/include \;
				fi
			done
		fi

	else
		cp -v $PROJECT_DIR/innosrvkm/services/server/env/linux/innogpu_drm_without_dpu.c $TARGET/innosrvkm/innogpu_drm_without_dpu.c
	fi
}

function copy_innoapu_open_srcs() {
	local module=innoapu
	local submodule_array=("img_mem" "vha")

	mkdir -p $TARGET_APU/innoapu && mkdir -p $TARGET_APU/innoapu/include && mkdir -p $TARGET_APU/innoapu/vha && mkdir -p $TARGET_APU/innoapu/img_mem

	for submodule in "${submodule_array[@]}"
	do
		for i in $(cat $DKMS_BUILD_DIR/${module}_${submodule}${OPEN_SRCS_SUFFIX})
		do
			if [[ $i =~ "/" ]]; then
				DIR=${i%/*}
				FILE=${i##*/}
				mkdir -p $TARGET_APU/${module}/include/$DIR
				find $PROJECT_DIR/${module} -name $FILE -not -path "*innoapu/vha/multi/*" -exec cp -v {} $TARGET_APU/${module}/include/$DIR \;
			elif [[ $i =~ ".c" ]]; then
				find $PROJECT_DIR/${module}/${submodule} -name $i  -not -path "*innoapu/vha/multi/*" -exec cp -v {} $TARGET_APU/${module}/${submodule} \;
			else
				find $PROJECT_DIR/${module}/ -name $i -not -path "*innoapu/vha/multi/*" -exec cp -v {} $TARGET_APU/${module}/include \;
			fi
		done
	done
}

function copy_innovpu_open_srcs() {
	copy_open_srcs innovpu
}

function copy_innosr_open_srcs() {
	copy_open_srcs innosr
}

function copy_innopower_open_srcs() {
	copy_open_srcs innopower
}

function generate_bin_innogpu() {
	generate_bin innogpu
}

function generate_bin_innodma() {
	generate_bin innodma
}

function generate_bin_innoion() {
	generate_bin innoion
}

function generate_bin_innopmbus() {
	generate_bin innopmbus
}

function generate_bin_innosrvkm() {
	generate_bin innosrvkm
}

function generate_bin_innovpu() {
	generate_bin innovpu
}

function generate_bin_innosr() {
	generate_bin innosr
}

function generate_bin_innopower() {
	generate_bin innopower
}

function generate_bin_innoapu() {
	local module=innoapu
	local submodule_array=("img_mem" "vha")

	for submodule in "${submodule_array[@]}"
	do
		objs_dir=$PROJECT_DIR/${module}/${submodule}/
		bin_name=$TARGET_APU/${module}/${submodule}/${submodule}_part.o_shipped
		bin_exclude_objs=$DKMS_BUILD_DIR/${module}_${submodule}${BIN_EXCLUDE_OBJ_SUFFIX}

		cnt=$(find $objs_dir -name "*.o" -not -name "${submodule}".o -not -name "${submodule}".mod.o | grep -vFf $bin_exclude_objs | wc -l)

		if [[ $cnt != 0 ]]; then
			$LD -r -o $bin_name `find $objs_dir -name "*.o" -not -name "${submodule}".o -not -name "${submodule}".mod.o | \
				grep -vFf $bin_exclude_objs`
		fi
	done
}

function make_module() {
	local module=$1
	prepare_${module}_exclude_objs
	copy_${module}_open_srcs
	generate_makefile_${module}
	generate_bin_${module}
}

function make_all_modules() {
	mkdir -p $TARGET

	if [[ $CONFIG_DRM_INNO_APU = "y" ]]; then
		mkdir -p $TARGET_APU
	fi

	make_module innogpu
	make_module innosrvkm
	make_module innodma

	if [[ $CONFIG_DRM_INNO_VPU = "y" ]]; then
		make_module innovpu
	fi

	if [[ $CONFIG_DRM_INNO_SR = "y" ]]; then
		make_module innosr
	fi

	if [[ $CONFIG_INNO_ION = "y" ]]; then
		make_module innoion
	fi

	if [[ $CONFIG_DRM_INNO_PMBUS = "y" ]]; then
		make_module innopmbus
	fi

	if [[ $CONFIG_DRM_INNO_POWER = "y" ]]; then
		make_module innopower
	fi

	if [[ $CONFIG_DRM_INNO_APU = "y" ]]; then
		make_module innoapu
	fi

	if [[ $CONFIG_DRM_INNO_DEBUG = "y" ]]; then
		mkdir -p $TARGET/tools
		cp -r $PROJECT_DIR/tools/gpu_info $TARGET/tools
		rm -rf $TARGET/tools/gpu_info/*.o
		cp $PROJECT_DIR/tools/Makefile $TARGET/tools
		if [[ $CONFIG_DRM_INNO_APU = "y" ]]; then
			mkdir -p $TARGET_APU/tools
		fi
	fi

	cp $DKMS_BUILD_DIR/Makefile $TARGET
	cp $PROJECT_DIR/Kbuild $TARGET
	sed -i '/innonulldisp/d' $TARGET/Kbuild
	cp $DKMS_BUILD_DIR/dkms.conf $TARGET
	mv $DKMS_BUILD_DIR/modules_config.sh $TARGET/
	cp $DKMS_BUILD_DIR/cfg_detect.sh $TARGET
	cp $DKMS_BUILD_DIR/test_item.sh $TARGET

	cp -r $DKMS_BUILD_DIR/kernel_api_support_test $TARGET/tools
	sed -i '/^LINK_TEST_DIR/c LINK_TEST_DIR="./tools/kernel_api_support_test"' $TARGET/cfg_detect.sh

	if [[ $CONFIG_DRM_INNO_APU = "y" ]]; then
		cp $DKMS_BUILD_DIR/Makefile $TARGET_APU/Makefile
		cp $PROJECT_DIR/Kbuild $TARGET_APU
		cp $DKMS_BUILD_DIR/dkms.conf $TARGET_APU
		cp $TARGET/modules_config.sh $TARGET_APU/
		cp $DKMS_BUILD_DIR/cfg_detect.sh $TARGET_APU
		cp $DKMS_BUILD_DIR/test_item.sh $TARGET_APU
		cp -r $DKMS_BUILD_DIR/kernel_api_support_test $TARGET_APU/tools
		cp $TARGET/cfg_detect.sh $TARGET_APU/
		sed -i '/innonulldisp/d' $TARGET_APU/Kbuild
		sed -i '/innosrvkm/ s/.*/	for dir in "innoapu\/img_mem" "innoapu\/vha"; \\/' $TARGET_APU/Makefile
		sed -i '/.o.cmd/ s/.*/		if [ -e $${dir} ];then touch $${dir}\/.$${dir##*\/}_part.o.cmd; fi \\/' $TARGET_APU/Makefile
		sed -i '/kernel_autocfg.h/Id' $TARGET_APU/Makefile
		sed -i "/KERNELDIR ?=/i\DKMS_VERSION ?= ${DKMS_VERSION}" $TARGET_APU/Makefile
		sed -i "/bash cfg_detect.sh/a\	cd /usr/src/innogpu-kernel-${DKMS_VERSION} && \$(MAKE)" $TARGET_APU/Makefile
		sed -i "/clean:/i\	cp innoapu/img_mem/img_mem.ko .\/" $TARGET_APU/Makefile
		sed -i "/clean:/i\	cp innoapu/vha/vha.ko .\/" $TARGET_APU/Makefile
		sed -i "/install:/i\	cd /usr/src/innogpu-kernel-${DKMS_VERSION} && \$(MAKE) clean" $TARGET_APU/Makefile
	fi

	if [ -n "$DDK_GTEST_PATH" ]; then
		cp -r $PROJECT_DIR/tools/dma_gtest $TARGET/tools
		rm $TARGET/tools/dma_gtest/dmatest
	fi
}

function set_dkms_version() {
	version=$1
	sed -i "s/PACKAGE_VERSION=\"2.1\"/PACKAGE_VERSION=\"${DKMS_VERSION}\"/" $TARGET/dkms.conf

	if [[ $CONFIG_DRM_INNO_APU = "y" ]]; then
		sed -i "s/PACKAGE_VERSION=\"2.1\"/PACKAGE_VERSION=\"${DKMS_VERSION}\"/" $TARGET_APU/dkms.conf
		sed -i '/PACKAGE_NAME/ s/.*/PACKAGE_NAME="innoapu-kernel"/' $TARGET_APU/dkms.conf
		sed -i '/BUILT_MODULE_NAME\[0\]/ s/.*/BUILT_MODULE_NAME[0]="img_mem"/' $TARGET_APU/dkms.conf
		sed -i "/AUTOINSTALL/i\BUILT_MODULE_NAME[1]=\"vha\"" $TARGET_APU/dkms.conf
		sed -i "/AUTOINSTALL/i\DEST_MODULE_LOCATION[1]=\"/kernel/drivers/gpu/drm/innogpu/\"" $TARGET_APU/dkms.conf
	fi
}

function add_dkms_module() {
	name=$1
	cnt=$2
	sed -i "/AUTOINSTALL/i\BUILT_MODULE_NAME[${cnt}]=\"${name}\"" $TARGET/dkms.conf
	sed -i "/AUTOINSTALL/i\DEST_MODULE_LOCATION[${cnt}]=\"/kernel/drivers/gpu/drm/innogpu/\"" $TARGET/dkms.conf
}

function generate_dkms_conf() {
	set_dkms_version $DKMS_VERSION
	if [[ $SEPARATE_BUILD != "y" ]]; then
		return 0
	fi

	cnt=1
	add_dkms_module "innodma" $cnt
	let cnt=$cnt+1

	add_dkms_module "innosrvkm" $cnt
	let cnt=$cnt+1

	if [[ $CONFIG_DRM_INNO_VPU = "y" ]]; then
		add_dkms_module "innovpu" $cnt
		let cnt=$cnt+1
	fi

	if [[ $CONFIG_DRM_INNO_SR = "y" ]]; then
		add_dkms_module "innosr" $cnt
		let cnt=$cnt+1
	fi

	if [[ $CONFIG_INNO_ION = "y" ]]; then
		add_dkms_module "innoion" $cnt
		let cnt=$cnt+1
	fi

	if [[ $CONFIG_DRM_INNO_PMBUS = "y" ]]; then
		add_dkms_module "innopmbus" $cnt
		let cnt=$cnt+1
	fi

	if [[ $CONFIG_DRM_INNO_POWER = "y" ]]; then
		add_dkms_module "innopower" $cnt
		let cnt=$cnt+1
	fi

	if [[ $CONFIG_DRM_INNO_DEBUG = "y" ]]; then
		add_dkms_module "innogpu_info" $cnt
		let cnt=$cnt+1
	fi
}

function config_rpm_package() {
	mkdir -p $HOME/rpmbuild/BUILD
	mkdir -p $HOME/rpmbuild/BUILDROOT
	mkdir -p $HOME/rpmbuild/RPMS
	mkdir -p $HOME/rpmbuild/SOURCES
	mkdir -p $HOME/rpmbuild/SPECS
	mkdir -p $HOME/rpmbuild/SRPMS
	echo 'RPM_VERSION' ${RPM_VERSION}
	echo 'RELEASE_VERSION' ${RPM_RELEASE}
	if [[ $1 = $GPU_RPM_PACKAGE_NAME ]]; then
		sed -i "s/Version:.*/Version:\t${RPM_VERSION}/g" $DKMS_BUILD_DIR/innogpu.spec.cache
		sed -i "s/Release:.*/Release:\t${RPM_RELEASE}/g" $DKMS_BUILD_DIR/innogpu.spec.cache
		sed -i "s/Name:.*/Name:\t${GPU_RPM_PACKAGE_NAME}/g"  $DKMS_BUILD_DIR/innogpu.spec.cache
	elif [[ $1 = $APU_RPM_PACKAGE_NAME ]]; then
		sed -i "s/Version:.*/Version:\t${RPM_VERSION}/g" $DKMS_BUILD_DIR/innoapu.spec.cache
		sed -i "s/Release:.*/Release:\t${RPM_RELEASE}/g" $DKMS_BUILD_DIR/innoapu.spec.cache
		sed -i "s/Name:.*/Name:\t${APU_RPM_PACKAGE_NAME}/g"  $DKMS_BUILD_DIR/innoapu.spec.cache
		sed -i "s|drm/innogpu|drm/innogpu/{img_mem.ko,vha.ko}|g" $DKMS_BUILD_DIR/innoapu.spec.cache
	fi
}

function build_rpm() {
	RPM_VERSION=${1:-'1.0'}
	RPM_RELEASE=${2:-'1.0'}
	if [[ $3 = $GPU_RPM_PACKAGE_NAME ]]; then
		echo "GPU_RPM_PACKAGE_NAME:  $GPU_RPM_PACKAGE_NAME"
		config_rpm_package $3
		cp -r ${GPU_RPM_PACKAGE_NAME}.tar $HOME/rpmbuild/SOURCES/
		cp -r $DKMS_BUILD_DIR/innogpu.spec.cache $HOME/rpmbuild/SPECS/innogpu.spec
	elif [[ $3 = $APU_RPM_PACKAGE_NAME ]]; then
		echo "APU_RPM_PACKAGE_NAME:  $APU_RPM_PACKAGE_NAME"
		config_rpm_package $3
		cp -r ${APU_RPM_PACKAGE_NAME}.tar $HOME/rpmbuild/SOURCES/
		cp -r $DKMS_BUILD_DIR/innoapu.spec.cache $HOME/rpmbuild/SPECS/innoapu.spec
	fi
}

function clean_rpm_proj() {
	rm -r -f $HOME/rpmbuild/BUILD
	rm -r -f $HOME/rpmbuild/BUILDROOT
	rm -r -f $HOME/rpmbuild/RPMS
	rm -r -f $HOME/rpmbuild/SOURCES
	rm -r -f $HOME/rpmbuild/SPECS
	rm -r -f $HOME/rpmbuild/SRPMS
}

function make_dkms_deb() {
	cd $TARGET

	if [[ $1 == 'deb' ]]; then
		if [ -n "$DDK_GTEST_PATH" ]; then
			echo "BUILT_MODULE_NAME[1]="innodma_gtest"" >> dkms.conf
			echo "DEST_MODULE_LOCATION[1]="/kernel/drivers/gpu/drm/innodma_gtest/"" >> dkms.conf
			echo "export CONFIG_GTEST_INNO_DMA=y" >> modules_config.sh
		fi
		dkms mkdeb --source-only
	elif [[ $1 == 'rpm' ]]; then
		cp $DKMS_BUILD_DIR/innogpu.spec $DKMS_BUILD_DIR/innogpu.spec.cache > /dev/null
		tar cvf ${PROJECT_DIR}/${GPU_RPM_PACKAGE_NAME}.tar * > /dev/null
		cd $PROJECT_DIR
		build_rpm ${DKMS_VERSION} ${DKMS_RELEASE_VERSION} ${GPU_RPM_PACKAGE_NAME}
		rpmbuild -bb $HOME/rpmbuild/SPECS/innogpu.spec
		cp /root/rpmbuild/RPMS/`arch`/${GPU_RPM_PACKAGE_NAME}*.rpm  $PROJECT_DIR
	else
		echo 'input error'
	fi

	if [[ $CONFIG_DRM_INNO_APU = "y" ]]; then
		cd $TARGET_APU
	if [[ $1 == 'deb' ]]; then
		dkms mkdeb --source-only
	elif [[ $1 == 'rpm' ]]; then
		clean_rpm_proj
		cp $DKMS_BUILD_DIR/innogpu.spec $DKMS_BUILD_DIR/innoapu.spec.cache > /dev/null
		tar cvf ${PROJECT_DIR}/${APU_RPM_PACKAGE_NAME}.tar * > /dev/null
		cd $PROJECT_DIR
		build_rpm ${DKMS_VERSION} ${DKMS_RELEASE_VERSION} ${APU_RPM_PACKAGE_NAME}
		rpmbuild -bb $HOME/rpmbuild/SPECS/innoapu.spec
		cp /root/rpmbuild/RPMS/`arch`/${APU_RPM_PACKAGE_NAME}*.rpm  $PROJECT_DIR
	else
		echo 'input error'
	fi
	fi
}

# 清理
function clean() {
	rm -rf $DKMS_BUILD_DIR/*.tmp
	cd $PROJECT_DIR
	make clean
	rm $HOME/rpmbuild -rf
	rm ${PROJECT_DIR}/${GPU_RPM_PACKAGE_NAME}.tar -rf
	rm $DKMS_BUILD_DIR/innogpu.spec.cache -rf
	if [[ $CONFIG_DRM_INNO_APU = "y" ]]; then
		rm ${PROJECT_DIR}/${APU_RPM_PACKAGE_NAME}.tar -rf
		rm $DKMS_BUILD_DIR/innoapu.spec.cache -rf
	fi
}
PACKAGE_TYPE_VALID=('rpm' 'deb')
if [[ ! "${PACKAGE_TYPE[@]}" =~ "${PACKAGE_TYPE}" ]]; then
	echo 'package type input error!!'
	echo 'Usage:  ./build  (rpm|deb) [version]'
	exit
fi
prepare_env_and_make
make_all_modules
generate_kbuild
generate_dkms_conf
make_dkms_deb ${PACKAGE_TYPE}
clean
